/*
 * P2040 CPU PCIe Driver Interface -- Only one device
 * 
 * Copyright 2016 VadaTech Incorporated.  All rights reserved.
 *
 */

 
 // Define this for different kernel 
#define LINUX_VERSION_CODE 196659 	// 3.0.51 for QorQ 1.7 SDK)
 
#define KERNEL_VERSION(a,b,c) (((a) << 16) + ((b) << 8) + (c))

// #include <linux/version.h>

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/pci.h>
#include <linux/init.h>
#include <linux/types.h>
#include <linux/fs.h>
#include <linux/cdev.h>
#include <linux/interrupt.h>
#include <linux/spinlock.h>
#include <linux/completion.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,26)
#include <asm/semaphore.h>
#include <asm/uaccess.h>
#else
#include <linux/semaphore.h>
#include <linux/uaccess.h>
#endif
#include <linux/wait.h>
#include <linux/sched.h>
#include <linux/poll.h>
#include <linux/mm.h>
#include <linux/delay.h>
#include <linux/dma-mapping.h>

#include "vt_simple_pcie_drv.h"

#define DRIVER_VERSION			"1.0"
#define DRIVER_NAME			"p2040_cpu_pcie_drv"

#define PCI_VENDOR_ID_VADATECH		0xABCD
#define PCI_DEVICE_ID_VADATECH_AMC516	0x5516
#define PCI_DEVICE_ID_VADATECH_AMC517	0x5517
#define PCI_DEVICE_ID_VADATECH_AMC525	0x5525

static dev_t				dev_id;
struct class* 				p2040_pcie_drv_class 	= NULL;
struct device*				p2040_pcie_drv_device 	= NULL;
static int				next_minor 		= 0;
void*					mmap_base 		= NULL;

#define CARD_FROM_INODE( inode )	(container_of((inode)->i_cdev, card_t, dev))

// removed from semaphore.h in 2.6.37
#ifndef init_MUTEX
#define init_MUTEX(sem) sema_init(sem, 1)
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,26)
static const __devinitdata struct pci_device_id pci_ids[] =
{
#else
DEFINE_PCI_DEVICE_TABLE(pci_ids) =
{
#endif
	{
		.vendor		= PCI_VENDOR_ID_VADATECH,
		.device		= PCI_DEVICE_ID_VADATECH_AMC516,
		.subvendor	= PCI_VENDOR_ID_VADATECH,
		.subdevice	= PCI_DEVICE_ID_VADATECH_AMC516
	},
	{
		.vendor		= PCI_VENDOR_ID_VADATECH,
		.device		= PCI_DEVICE_ID_VADATECH_AMC517,
		.subvendor	= PCI_VENDOR_ID_VADATECH,
		.subdevice	= PCI_DEVICE_ID_VADATECH_AMC517
	},
	{
		.vendor		= PCI_VENDOR_ID_VADATECH,
		.device		= PCI_DEVICE_ID_VADATECH_AMC525,
		.subvendor	= PCI_VENDOR_ID_VADATECH,
		.subdevice	= PCI_DEVICE_ID_VADATECH_AMC525
	},
	{ 0 }
};


typedef struct card_s
{
	struct pci_dev*			pci_dev;
	char				pci_name[32];
	dev_t				dev_id;
	struct cdev			dev;
	int				got_dev;
} card_t;

static int open( struct inode *inode, struct file *file )
{
	return 0;
}

static int close( struct inode *inode, struct file *file )
{
	return 0;
}

static unsigned int poll( struct file* file, poll_table* wait )
{
	return 0;
}

static int ioc_get_reg32(void __user* arg)
{
	reg_t reg;

	if( __copy_from_user( (void*)&reg, arg, sizeof( reg ) ) )
	{
		return -EFAULT;
	}

	reg.data = ioread32(mmap_base + reg.addr);

	if( __copy_to_user( arg, (const void*)&reg, sizeof( reg ) ) )
	{
		return -EFAULT;
	}

	return 0;
}

static int ioc_set_reg32(void __user* arg)
{
	reg_t reg;

	if( __copy_from_user( (void*)&reg, arg, sizeof( reg ) ) )
	{
		return -EFAULT;
	}

	iowrite32(reg.data, mmap_base + reg.addr);

	return 0;
}

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,36)
static int ioctl( struct inode *inode,
		  struct file *file,
		  unsigned int command,
		  unsigned long arg )
#else
static long ioctl( struct file *file,
		   unsigned int command,
		   unsigned long arg )
#endif
{
	int retval = -ENOTTY;

	// Check that it is destined for us
	if( _IOC_TYPE(command) != IOC_DEVICE_MAGIC ||
		_IOC_NR(command) > IOC_MAXNR )
	{
		printk( KERN_INFO "ioctl magic = %x; num = %x\n", _IOC_TYPE(command), _IOC_NR(command));
		goto out;
	}

	// Check memory access permissions based on transfer direction
	if( _IOC_DIR(command) & _IOC_READ)
	{
		// Access for write also implies access for read
		if( !access_ok( VERIFY_WRITE, (void __user*)arg, _IOC_SIZE(command)) )
		{
			retval = -EFAULT;
			printk( KERN_INFO "ioctl read permissions failed\n");
			goto out;
		}
	}
	else if( _IOC_DIR(command) & _IOC_WRITE )
	{
		if( !access_ok( VERIFY_READ, (void __user*)arg, _IOC_SIZE(command)) )
		{
			retval = -EFAULT;
			printk( KERN_INFO "ioctl write permissions failed\n");
			goto out;
		}
	}

	// see which command it is
	switch( command )
	{
	case IOC_GET_REG32:
		retval = ioc_get_reg32( (void __user*)arg );
		break;
	case IOC_SET_REG32:
		retval = ioc_set_reg32( (void __user*)arg );
		break;
	default:
		retval = -ENOIOCTLCMD;
		break;
	}

out:
	return retval;
}

static struct file_operations card_fops =
{
	.owner			= THIS_MODULE,
	.llseek			= no_llseek,
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,36)
	.ioctl			= ioctl,
#else
	.unlocked_ioctl		= ioctl,
#endif
	.open			= open,
	.poll			= poll,
	.release		= close,
};

static void cleanup_card( struct pci_dev* pci_dev )
{
	card_t* card = (card_t*)pci_get_drvdata( pci_dev );

	if( !card )
	{
		return;
	}

	device_destroy (p2040_pcie_drv_class, card->dev_id);
		
	// Unregister dev node
	if( card->got_dev ) {
		card->got_dev = 0;
		cdev_del( &card->dev );
	}
	
	/* unmap the PCI memory space */
	if(mmap_base) 
	{
		iounmap( mmap_base );
	}

	/* disable the PCI device */
	pci_disable_device( pci_dev );

	/* free the card structure and unlink */
	pci_set_drvdata( pci_dev, NULL );
	kfree( card );
}

static int 
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,0,51)
__devinit 
#endif
probe( struct pci_dev* pci_dev,	const struct pci_device_id* pci_id )
{
	int retval;
	card_t* card;

	/* allocate our private card structure */
	card = (card_t*)kmalloc( sizeof( card_t ), GFP_KERNEL );
	if( !card )
	{
		retval = -ENOMEM;
		goto out;
	}

	/* Clear the card structure, set it up, and cross link it */
	memset( (void*)card, 0, sizeof( card_t ) );

	sprintf( card->pci_name, "%s-%02u:%02u.%u",
			 DRIVER_NAME,
			 pci_dev->bus->number,
			 PCI_SLOT(pci_dev->devfn),
			 PCI_FUNC(pci_dev->devfn) );
	printk( KERN_INFO "%s: probing\n", card->pci_name );
	card->pci_dev = pci_dev;

	pci_set_drvdata( pci_dev, (void*)card );

	/* Enable the device */
	retval = pci_enable_device( pci_dev );
	if( retval )
	{
		goto cleanup;
	}

	/* Setup the BAR */
	if( !(pci_resource_flags( pci_dev, 0 ) & IORESOURCE_MEM) )
	{
		printk( KERN_ERR "%s: Missing BAR0\n", card->pci_name );
		retval = -ENODEV;
		goto cleanup;
	}

	printk( KERN_INFO "%s: Mapping BAR0 (0x%08lx-0x%08lx %luKB)\n",
			card->pci_name,
			(unsigned long)pci_resource_start(pci_dev, 0),
			(unsigned long)pci_resource_end(pci_dev, 0),
			(unsigned long)pci_resource_len(pci_dev, 0) / 1024 );

	mmap_base = ioremap_nocache( pci_resource_start( pci_dev, 0 ),
					pci_resource_len( pci_dev, 0 ) );
	if( !mmap_base )
	{
		printk( KERN_ERR "Failed to map pcie bar\n" );
		retval = -ENOMEM;
		goto cleanup;
	}

	/* allocate device node and character device */
	card->dev_id = MKDEV( MAJOR(dev_id), next_minor++ );
	
	/* something wrong must be happened 
	 * there's no way to have two more devices probed 
	 */ 
 
	if(next_minor >= 2) 
	{
		retval = -ENOMEM;
		goto cleanup;		
	}

	p2040_pcie_drv_device = device_create (p2040_pcie_drv_class, NULL, card->dev_id, NULL, "vt_simple_pcie_drv");

	if( IS_ERR(p2040_pcie_drv_device) ) {
		printk( KERN_ERR "%s: Unable to create device\n", DRIVER_NAME );
		retval = PTR_ERR(p2040_pcie_drv_device);
		goto cleanup;
	}
	
	// Add the character device (after which the user can start to use it)
	cdev_init( &card->dev, &card_fops );
	card->dev.owner = THIS_MODULE;
	retval = cdev_add( &card->dev, card->dev_id, 1);

	if( retval )
	{
		goto cleanup;
	}

	card->got_dev = 1;

	printk( KERN_INFO "%s: Card is device %d:%d\n",
		card->pci_name, MAJOR(card->dev_id), MINOR(card->dev_id) );
	

	retval = 0;
	goto out;

cleanup:
	cleanup_card( pci_dev );
out:
	return retval;
}

#if LINUX_VERSION_CODE < KERNEL_VERSION(3,0,51)
static void __devexit
#else
static void 
#endif
remove( struct pci_dev* pci_dev )
{
	cleanup_card( pci_dev );
}


static struct pci_driver pci_driver =
{
	.name		= DRIVER_NAME,
	.id_table	= pci_ids,
	.probe		= probe,
	.remove		= remove
};

static int __init drv_init( void )
{
	int		retval;

	printk( KERN_INFO "%s: driver version %s\n",
			DRIVER_NAME, DRIVER_VERSION );

	/* allocate a new dynamic major number */
	retval = alloc_chrdev_region( &dev_id, 0, 1, DRIVER_NAME );

	if( retval )
	{
		printk( KERN_ERR "%s: Unable to allocate dynamic major device number\n", DRIVER_NAME );
		goto out;
	}

	p2040_pcie_drv_class = class_create (THIS_MODULE, "vt_pcie");
	
	if( IS_ERR(p2040_pcie_drv_class) ) {
		printk( KERN_ERR "%s: Unable to create device class\n", DRIVER_NAME );
		retval = PTR_ERR(p2040_pcie_drv_class);
		goto unalloc_exit;
	}
	
	retval = pci_register_driver( &pci_driver );

	if( retval )
	{
		printk( KERN_ERR "%s: Unable to register as a PCI driver\n", DRIVER_NAME );
		goto class_unregister;
	}

	retval = 0;
	goto out;

class_unregister:
	class_unregister(p2040_pcie_drv_class);
	class_destroy(p2040_pcie_drv_class);
	
unalloc_exit:
	unregister_chrdev_region( dev_id, 1 );
out:
	return retval;
}

static void __exit drv_exit( void )
{
	pci_unregister_driver( &pci_driver );
	class_unregister(p2040_pcie_drv_class);
	class_destroy (p2040_pcie_drv_class);	
	unregister_chrdev_region( dev_id, 1 );
}

module_init( drv_init );
module_exit( drv_exit );
MODULE_LICENSE("GPL");
